/*
 * GlushkovIndexate.cpp
 *
 *  Created on: 14. 4. 2016
 *	  Author: Tomas Pecka
 */

#include "GlushkovIndexate.h"

#include <rte/formal/FormalRTEElements.h>

#include <common/DefaultSymbolsPairType.h>

namespace rte {

// ----------------------------------------------------------------------------

common::ranked_symbol < > GlushkovIndexate::getSymbolFromGlushkovPair ( const common::ranked_symbol < > & symbol ) {
	const ext::pair < DefaultSymbolType, DefaultSymbolType > & sps = ( ( const DefaultSymbolsPairType & ) symbol.getSymbol ( ).getData ( ) );
	return common::ranked_symbol < > ( sps.first, symbol.getRank ( ) );
}

FormalRTE < > GlushkovIndexate::index ( const rte::FormalRTE < > & rte ) {
	int i = 1;

	return FormalRTE < > ( FormalRTEStructure < DefaultSymbolType, primitive::Unsigned > ( rte.getRTE ( ).getStructure ( ).accept < ext::ptr_value < rte::FormalRTEElement < DefaultSymbolType, primitive::Unsigned > >, GlushkovIndexate::Formal > ( i ) ) );
}

// ----------------------------------------------------------------------------

ext::ptr_value < FormalRTEElement < DefaultSymbolType, primitive::Unsigned > > GlushkovIndexate::Formal::visit ( const rte::FormalRTESymbolAlphabet < DefaultSymbolType, primitive::Unsigned > & node, int & i ) {
	DefaultSymbolsPairType sps = DefaultSymbolsPairType ( ext::make_pair ( DefaultSymbolType ( node.getSymbol ( ).getSymbol ( ) ), DefaultSymbolType ( i++ ) ) );
	ext::ptr_vector < rte::FormalRTEElement < DefaultSymbolType, primitive::Unsigned > > children;
	for ( const rte::FormalRTEElement < DefaultSymbolType, primitive::Unsigned > & e : node.getElements ( ) ) {
		children.push_back ( ( FormalRTEElement < DefaultSymbolType, primitive::Unsigned > && ) e.accept < ext::ptr_value < rte::FormalRTEElement < DefaultSymbolType, primitive::Unsigned > >, GlushkovIndexate::Formal > ( i ) );
	}

	return ext::ptr_value < FormalRTESymbolAlphabet < DefaultSymbolType, primitive::Unsigned > > ( FormalRTESymbolAlphabet < DefaultSymbolType, primitive::Unsigned > ( common::ranked_symbol < > ( DefaultSymbolType ( sps ), node.getSymbol ( ).getRank ( ) ), std::move ( children ) ) );
}

ext::ptr_value < FormalRTEElement < DefaultSymbolType, primitive::Unsigned > > GlushkovIndexate::Formal::visit ( const rte::FormalRTESymbolSubst < DefaultSymbolType, primitive::Unsigned > & node, int & ) {
	return ext::ptr_value < FormalRTEElement < DefaultSymbolType, primitive::Unsigned > > ( FormalRTESymbolSubst < DefaultSymbolType, primitive::Unsigned > ( node ) );
}

ext::ptr_value < FormalRTEElement < DefaultSymbolType, primitive::Unsigned > > GlushkovIndexate::Formal::visit ( const rte::FormalRTEAlternation < DefaultSymbolType, primitive::Unsigned > & node, int & i ) {
	ext::ptr_value < rte::FormalRTEElement < DefaultSymbolType, primitive::Unsigned > > left = node.getLeftElement ( ).accept < ext::ptr_value < rte::FormalRTEElement < DefaultSymbolType, primitive::Unsigned > >, GlushkovIndexate::Formal > ( i );
	ext::ptr_value < rte::FormalRTEElement < DefaultSymbolType, primitive::Unsigned > > right = node.getRightElement ( ).accept < ext::ptr_value < rte::FormalRTEElement < DefaultSymbolType, primitive::Unsigned > >, GlushkovIndexate::Formal > ( i );
	return ext::ptr_value < FormalRTEElement < DefaultSymbolType, primitive::Unsigned > > ( FormalRTEAlternation < DefaultSymbolType, primitive::Unsigned > ( std::move ( left ), std::move ( right ) ) );
}

ext::ptr_value < FormalRTEElement < DefaultSymbolType, primitive::Unsigned > > GlushkovIndexate::Formal::visit ( const rte::FormalRTESubstitution < DefaultSymbolType, primitive::Unsigned > & node, int & i ) {
	ext::ptr_value < rte::FormalRTEElement < DefaultSymbolType, primitive::Unsigned > > left = node.getLeftElement ( ).accept < ext::ptr_value < rte::FormalRTEElement < DefaultSymbolType, primitive::Unsigned > >, GlushkovIndexate::Formal > ( i );
	ext::ptr_value < rte::FormalRTEElement < DefaultSymbolType, primitive::Unsigned > > right = node.getRightElement ( ).accept < ext::ptr_value < rte::FormalRTEElement < DefaultSymbolType, primitive::Unsigned > >, GlushkovIndexate::Formal > ( i );
	return ext::ptr_value < FormalRTEElement < DefaultSymbolType, primitive::Unsigned > > ( FormalRTESubstitution < DefaultSymbolType, primitive::Unsigned > ( std::move ( left ), std::move ( right ), node.getSubstitutionSymbol ( ) ) );
}

ext::ptr_value < FormalRTEElement < DefaultSymbolType, primitive::Unsigned > > GlushkovIndexate::Formal::visit ( const rte::FormalRTEIteration < DefaultSymbolType, primitive::Unsigned > & node, int & i ) {
	ext::ptr_value < rte::FormalRTEElement < DefaultSymbolType, primitive::Unsigned > > element = node.getElement ( ).accept < ext::ptr_value < rte::FormalRTEElement < DefaultSymbolType, primitive::Unsigned > >, GlushkovIndexate::Formal > ( i );
	return ext::ptr_value < FormalRTEElement < DefaultSymbolType, primitive::Unsigned > > ( FormalRTEIteration < DefaultSymbolType, primitive::Unsigned > ( std::move ( element ), node.getSubstitutionSymbol ( ) ) );
}

ext::ptr_value < FormalRTEElement < DefaultSymbolType, primitive::Unsigned > > GlushkovIndexate::Formal::visit ( const rte::FormalRTEEmpty < DefaultSymbolType, primitive::Unsigned > &, int & ) {
	return ext::ptr_value < FormalRTEElement < DefaultSymbolType, primitive::Unsigned > > ( FormalRTEEmpty < DefaultSymbolType, primitive::Unsigned > ( ) );
}

// ----------------------------------------------------------------------------

} /* namespace rte */
